# -*- coding: utf-8 -*-

import re

#from django.conf import settings
#from django.contrib import admin
import django.forms as forms

import selectable.forms as select_forms

import contacts.lookups as c_lookups
import references.lookups as r_lookups
import contacts.models as contacts_models
import contacts.widgets as c_widgets
import references.models as ref_models

#from contacts.logger import logger


### Données et fonctions utilitaires

# Définition du widget utilisé pour les différentes adresse
# On le centralise ici pour qu'il soit partout pareil.
_widget_adresse = {
        'adresse': forms.Textarea(attrs={ 'rows': 3, 'cols': 40 }),
    }

### Form d'admin de modèles
class FormAssolement(forms.ModelForm):
    """
    Petite redéfinition du formulaire des assolements pour
    classer la liste de choix des cultures par catégories.
    """
    class Meta:
        model = contacts_models.Assolement

    def __init__(self, *args, **kwargs):
        """
        """
        super(FormAssolement, self).__init__(*args, **kwargs)
        self.fields['culture'].choices = self._get_cultures_as_choice()

    def _get_cultures_as_choice(self):
        """
        Retourne la liste des cultures organisée de façon à afficher les
        catégories.
        """
        choices = []
        for cat in ref_models.CATEGORIES_CULTURE:
            cultures = [
                ( c.id, c.designation.capitalize() )
                for c in ref_models.Culture.objects
                .filter(categorie=cat.lower()).order_by('indice_ordre') ]
            choices.append((cat.capitalize(), cultures))
        return choices


class FormAssolementRo(forms.ModelForm):
    class Meta:
        model = contacts_models.Assolement


class ContactAdminForm(forms.ModelForm):
    """
    Personnalisation du formulaire pour limiter l'adresse sur 4 lignes (pour
    simplifier la génération d'étiquettes) et affecter automatiquement les
    Cette classe n'est pas utilisée directement, mais est utilisable par
    héritage pour tous les modèles représentant un contact.
    """
    # La sous-classe Meta doit être définie par les classes filles pour
    # désigner le modèle qui s'applique. Meta n'est pas hérité.
    #
    #class Meta:
    #    model = ...
    #    widgets = _widget_adresse.copy()

    def clean_adresse(self):
        """
        Limite l'adresse à 4 lignes.
        """
        addr = re.sub("\s+$", "", self.cleaned_data["adresse"])
        if len(re.findall("\n", addr)) > 3:
            raise forms.ValidationError("L'adresse ne doit pas comporter"
                                        " plus de 4 lignes.")
        return addr

    def clean(self):
        """
        Affecte des valeurs par défaut au code postal et
        à la ville lorsque seule la commune est précisée.
        """
        commune = self.cleaned_data['commune']
        if commune:
            if not self.cleaned_data['code_postal']:
                self.cleaned_data['code_postal'] = commune.code_postal
            if not self.cleaned_data['ville']:
                self.cleaned_data['ville'] = commune.nom
        return self.cleaned_data


class StructureAdminForm(ContactAdminForm):
    """
    Réutilisation des méthodes de ContactAdminForm pour Structure.
    """
    class Meta:
        model = contacts_models.StructureRo
        widgets = _widget_adresse.copy()


class AgriculteurAdminForm(ContactAdminForm):
    """
    Personnalisation du formulaire pour limiter l'adresse sur 4 lignes (pour
    simplifier la génération d'étiquettes).
    """
    class Meta:
        model = contacts_models.Structure
        widgets = {
            'adresse': forms.Textarea(attrs={ 'rows': 3, 'cols': 40 }),
            'commune': select_forms.AutoCompleteSelectWidget(
                lookup_class=r_lookups.CommuneLookup),
            }


class PersonneAdminForm(ContactAdminForm):
    """ Personnalisation du formulaire pour limiter l'adresse sur 4
    lignes (pour simplifier la génération d'étiquettes)."""
    class Meta:
        model = contacts_models.Personne
        widgets = _widget_adresse.copy()


class RdvForm(forms.ModelForm):
    class Meta:
        model = contacts_models.Rdv
        widgets = {
            'personne_morale': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.StructureLookup),
            'organisateur': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.PersonneLookup),
            'intervenant': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.PersonneLookup),
            'lieu': select_forms.AutoCompleteSelectWidget(
                lookup_class=r_lookups.CommuneLookup),
            }

    def __init__(self, *args, **kwargs):
        super(RdvForm, self).__init__(*args, **kwargs)
        self._pm_pk = self._get_pm(kwargs.get('initial', {}))
        self._filter_projetbio()

    def _get_pm(self, initial):
        "lookup associated personne morale"
        pm_pk = (self.instance and self.instance.personne_morale
                 and self.instance.personne_morale.pk)
        pm_pk = pm_pk or initial.get('personne_morale')
        return pm_pk

    def _filter_projetbio(self):
        u"""restreint la liste des projets bio affichés en fonction de
        la personne morale si celle ci est indiquée, sinon affiche une
        liste vide"""
        fd_projet_bio = self.fields.get('projet_bio')
        # qs inchangée en cas de POST, si besoin validation, cf clean()
        if fd_projet_bio and not self.is_bound:
            qs_projet_bio = getattr(fd_projet_bio, 'queryset', None)
            pm_pk = self._pm_pk
            fd_projet_bio.queryset = (
                qs_projet_bio.filter(agriculteur=pm_pk) if pm_pk
                else qs_projet_bio.none())


# Pour le moment inutile car utilisation de
# radio_fields = {"field": admin.HORIZONTAL}
# mais peut etre utile hors de l'admin
class ProjetBioForm(forms.ModelForm):
    class Meta:
        model = contacts_models.ProjetBio

        widgets = {
            'pond_savoirfaire': c_widgets.RadioSelect,
            'pond_motivation': c_widgets.RadioSelect,
            'pond_entourage': c_widgets.RadioSelect,
            'pond_foncier': c_widgets.RadioSelect,
            'pond_materiel': c_widgets.RadioSelect,
            'pond_reglementation': c_widgets.RadioSelect,
            'pond_commercialisation': c_widgets.RadioSelect,
            'pond_financement': c_widgets.RadioSelect,
            'pond_maindoeuvre': c_widgets.RadioSelect,
            }
# EOF
